%{
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include "ast.h"
#include "parser.h"

int nl = 1;
void ERROR (const char * fmt, ...);
%}

%option noyywrap

D [0-9]
W [ \t\r]
ID [a-zA-Z][a-zA-Z0-9]*

%%

"print"   { return T_print; }
"let"     { return T_let; }
"begin"   { return T_begin; }
"end"     { return T_end; }
"for"     { return T_for; }
"do"      { return T_do; }

{ID}         { yylval.var = yytext[0]; return T_var; }
{D}+         { yylval.num = atoi(yytext); return T_const; }

[\+\-\*/=\(\)\,]  { return yytext[0]; }

{W}+    { /* nothing */ }
\n      { nl++; }

.       { ERROR("Illegal character: '%c'", yytext[0]); }

%%

void ERROR (const char * fmt, ...)
{
   va_list args;
   va_start(args, fmt);
   fprintf(stderr, "Line %d: ", nl);
   vfprintf(stderr, fmt, args);
   fprintf(stderr, "\n");
   va_end(args);
   exit(1);
}
